////////////////////////////////////////////////////////////////////////////////
//
//   RX_MODULAR_32C
//
//
//  USANDO PIC 12F675/629 PARA DECODIFICAO DOS SINAIS
//  (USA INTERRUPO DE MUDANA DE ESTADO DE GP3 E TIMER1 PARA ADQUIRIR TEMPOS)
//  COM 1 CANAL, SENDO CONFIGURAVEL EM MODO PULSO, RETENO USANDO UM JUMPER.
//  USE APENAS EM CONJUNTO COM O TX DE 32 CANAIS (ENVIA 7 BYTES)
//
//
//  COMPILADOR CCS C V3.2.
//
//  AUTOR:  CLUDIO LRIOS
//
//  INCIO: 25/05/2016
//
//  APLICAO: CONTROLAR ILUMINAO, BOMBAS, ETC.
//
//  FUNCIONAMENTO:
//  TRABALHA COM UM NICO TX DE 32 BOTES  E  RECEPTORES MODULARES DE 1 CANAL,
//  NA QUANTIDADE DE 32 RECEPTORES.
//  MAIS DE UM RECEPTOR PODE SER GRAVADO COM DETERMINADO BOTO DO TX.
//  UMA APLICAO SERIA LIGAR VRIOS DISPOSITIVOS EM UM UNICO ACIONAMENTO DE UM
//  BOTO.
//
//  PROGRAMAO DOS RECEPTORES:
//
//  1) APERTE O BOTO 'PROG' DO RECEPTOR.
//  2) IR LIGAR O LED 'PROG' PARA INDICAR MODO DE PROGRAMAO
//  3) ACIONE O TX APERTANDO O BOTO DESEJADO
//  4) O LED 'PROG' APAGAR INDICANDO QUE ACEITOU A PROGRAMAO E ACIONAR A
//     SADA.
//  5) SE NO HOUVER ACIONAMENTOS DO TX, OU SE NO RECEBEU O SINAL, O LED 'PROG'
//     IR APAGAR DEPOIS DE  20 SEGUNDOS.
//  6) COLOQUE O JUMPER P/ ESCOLHER  MODO 'PULSO' OU RETIRE PARA MODO 'RETENO'.
//  7) REPITA PARA CADA RECEPTOR OS PASSOS ACIMA.
//
//  NOTA: MAIS DE UM BOTO PODE SER GRAVADO EM CADA RECEPTOR.
//        VRIOS RECEPTORES PODEM SER GRAVADOS PARA ACIONAR COM UM BOTO OU MAIS
//
//  APAGAR BOTO:
//
//  APERTE O BOTO 'PROG' POR MAIS DE 5 SEGUNDOS AT O LED 'PROG' COMEAR A PISCAR.
//  QUANDO ISTO ACONTECER, A MEMRIA FOI LIMPA. REGRAVE O BOTO DESEJADO SEGUINDO
//  OS PASSOS ACIMA.
//
//  SUJEITO A BUGS AINDA NO OBSERVADOS EM USO. MONTAGEM REALIZADA E TESTADA
//  SOMENTE EM  PLACA DE PROTOBOARD.
//  ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG LARIOS.TECNOLOGIA.WS
//==============================================================================
//                 DEFINIOES DO USURIO
//==============================================================================
//ESCOLHA DO MICROCONTROLADOR:
// Para usar o PIC12F675, descomente o abaixo e comente para usar o 12F629


      #define  F675


//==============================================================================


//==============================================================================
//     NOTA IMPORTANTE  PARA OBTER O FUNCIONAMENTO DESTE SOFTWARE
//==============================================================================
// ATENO: CERTIFIQUE DE EXISTIR O BYTE DE CALIBRAO NO ENDEREO 3FFH DA FLASH
// Caso tenha sido apagado acidentalmente, ao ligar, aps a programao do pic,
// NO funcionar. O programa ir fazer um 'call'para buscar o byte de calibra-
// o, mas no encontrar a instruo de retorno 'retlw xx'. Isto far o progra-
// se perder, resetando continuamente.
// COMO SABER SE TENHO O BYTE DE CALIBRAO NO PIC?
// LENDO A FLASH COM UM PROGRAMADOR, DEVER SER ENCONTRADO NO ENDEREO 0X3FF, UM
// VALOR COMEANDO COM 34 SEGUIDO DE 2 DIGITOS. EX. 3480. (34=RETLW 80= LITERAL
// QUE SER RETORNADO EM 'W' PARA CARGA DO REGISTRADOR '0SCCAL'.



//      #define PERDI_BYTE_CALIBRACAO



//Descomente o acima se voc tem um pic que foi apagado o byte de calibrao.
//Ser substituido por valor padro que permita a operao do circuito.
//==============================================================================


//==============================================================================
//
//          DEFINIES GERAIS  DO  PROGRAMA
//
//==============================================================================
#ifdef   F675
#include <12F675.h>         // includes
#else
#include <12F629.h>         // includes
#endif
#rom 0x217F = {0}
#zero_ram
#ignore_warnings 203,202  //desconsidera mensagem de 'condition always true'

#fuses INTRC_IO,NOWDT,NOPROTECT, NOMCLR ,BROWNOUT, PUT  // palavra  configurao
#use delay(clock=4000000)  // clock para 4 mhz
#use fast_io(A)  //  sentido das portas feita pelo programador

#ifdef  PERDI_BYTE_CALIBRACAO  // calibrao do oscilador interno do pic
#ROM  0X3FF = {0X34FC}//mxima freq= 0x34fc ;mdia=0x3480; minima=0x3400
#endif


//bits
#bit    s0    = 0X05.5  // PINO 2
#bit    rfin   = 0x05.3  // PINO 4
#bit    led    = 0x05.1  // PINO 6
#bit    bot    = 0x05.0  // PINO 7
#bit    jump   = 0x05.4  // PINO 3
//------------------------------------------------------------------------------

//constantes

#define qbit   56;//56 bits de dados
const int8 TMAX_LED = 20;//tempo que led prog fica aceso apos aperto do botao
const int8 TMAX_SAIDAS_LIGADAS= 20;// em modo pulso

//FLAGS GLOBAIS
int1 flag_rok,flag_gr,flag_s0;

//REGISTRADORES GLOBAIS
int32 buffer1; //buffer de recepo
#locate buffer1=0x33 //necessrio para funcionamento dos ponteiros
int32 buffer; //buffer de recepo
#locate buffer=0x2f//necessrio para funcionamento dos ponteiros
int8 qb,tl1,tl2,tm1,tm2,tm3;//registradores globais
int16   t_basico;//TEMPO BSICO


//BYTES
#byte   TRISIO = 0X85
#byte   tmr0 = 0x01
#byte   GPIO = 0X05
#byte   tmr1l= 0x0e
#byte   tmr1h= 0x0f
#byte   wpu  = 0x95
//BIT
#bit    tmr1_if= 0x0c.0  //flag do tmr1 over flow


//==============================================================================
////////////////////////////////////////////////////////////////////////////////
//
//          SUBROTINAS E FUNES  DO PROGRAMA
//
////////////////////////////////////////////////////////////////////////////////
//==============================================================================



//==============================================================================
//   APAGAR EEPROM
//==============================================================================
void apagar_eeprom(){ int8 a; for(a=0;a<128;a++) write_eeprom(a,0xff); }

//==============================================================================
//   GRAVAR  TX RECEBIDO NA EEPROM
//==============================================================================
void grava_6bytes_eeprom (int8 address_init, int8 *pont){

// onde int_addres = endereo inicial da gravao
// *pont = endereo inicial da informao (6 bytes seguidos)

     write_eeprom(address_init + 0,*(pont));
     write_eeprom(address_init + 1,*(pont+1));
     write_eeprom(address_init + 2,*(pont+2));
     write_eeprom(address_init + 3,*(pont+3));
     write_eeprom(address_init + 4,*(pont+4));
     write_eeprom(address_init + 5,*(pont+5));

}

 //==============================================================================
//   LOCALIZA ENDEREO LIVRE NA EEPROM PARA GRAVAR TX RECEBIDO
//==============================================================================
int procura_6bytes_livres_eeprom () {

 int a;
   for (a=0;a<126;a+=6){  // copia buffer para sbuffer
     if(read_eeprom(a)!=0xff)continue;
     if(read_eeprom(a+1)!=0xff)continue;
     if(read_eeprom(a+2)!=0xff)continue;
     if(read_eeprom(a+3)!=0xff)continue;
     if(read_eeprom(a+4)!=0xff)continue;
     if(read_eeprom(a+5)!=0xff)continue;
     return  a;//achou 3 posies para gravar novo tx
      }
   return 0xff;//memria cheia
  }

//==============================================================================
//   LOCALIZA TX RECEBIDO SE GRAVADO NA EEPROM
//==============================================================================
int procura_6bytes_gravados_eeprom (int *pont) {

  int a;

  for (a=0;a<126;a+=6){  //
     if(read_eeprom(a)!=*(pont))continue;//se diferente pula e vai para novo ciclo
     if(read_eeprom(a+1)!=*(pont +1))continue;
     if(read_eeprom(a+2)!=*(pont+2))continue;
     if(read_eeprom(a+3)!=*(pont+3))continue;
     if(read_eeprom(a+4)!=*(pont+4))continue;
     if(read_eeprom(a+5)!=*(pont+5))continue;
     return  0xaa;//achou tx gravado
  }
  return 0xff;//no achou tx
 }

//==============================================================================
//   TX RECEBIDO ENCONTRADO NA EEPROM - ACIONAR SADAS
//==============================================================================

void  acionar  (){

   if(procura_6bytes_gravados_eeprom(&buffer+1)==0xaa){
   tl1=0; tl2=0;//reseta contadores de tempo de saidas ligadas
   flag_gr=0;//reseta flag de gravao

//==============================================================================
//   ACIONAMENTO DE SADA CONFORME O JUMPER - PULSO OU RETENO
//==============================================================================

 if(jump){
    if(!flag_s0){
      s0=!s0; write_eeprom(127,s0); flag_s0=1;}
      tl1=0;tl2=0;
    }
  else{ s0=1; }


 }


//==============================================================================
//   TESTA SE DEVE GRAVAR A RECEPO
//==============================================================================
 else{
    if(flag_gr){   //gravar na eeprom o buffer
        grava_6bytes_eeprom (procura_6bytes_livres_eeprom(),&buffer+1);
        flag_gr=0;
       }
     }


 flag_rok=0;  //libera nova recepo
 }

//==============================================================================
////////////////////////////////////////////////////////////////////////////////
//    ROTINA DE RECEPO  (INTERRUPO POR MUDANA DE ESTADO DE GP3)
////////////////////////////////////////////////////////////////////////////////
//==============================================================================
#int_ra

VOID isr(){


  gpio&=gpio;//necessita leitura para poder setar flag de mudana de estado
  if(flag_rok||rfin){return;} //tem recepo no buffer ou mudana de estado '1'
  //rfin=0  objetivo: localizar o intervalo para sincronismo de recepo
  set_timer1(~5000);

  tmr1_if=0;//apaga flag de over flow do timer 0
  while(!tmr1_if){ if(rfin) return;}//erro


   //rfin=0
  set_timer1(~15000);

  tmr1_if=0;//apaga flag de over flow do timer 0
   while(!tmr1_if&&!rfin);//aguarda rfin se tornar '1' ou estouro do tmr1
  if(tmr1_if) return; //erro; excessivamente alto >65ms


  //rfin=1 comea o pulso bsico inicial
  set_timer1(0);
  tmr1_if=0;
  while(rfin&&!tmr1_if);//aguarda completar pulso basico inicial
  if(tmr1_if) return; //pulso 1 excessivamente alto >65ms
  //rfin=0
  t_basico=get_timer1(); // obtm o pulso bsico

 //janela de aceitao do pulso bsico inicial
  if((t_basico<200)||(t_basico>1000)) return; //erro no foi pulso basico
  qb=qbit;//ajusta quantidade de bits a receber
  buffer=0;//xxx
  buffer1=0;//xxxx
  do{//lao de repetio 'do'
      set_timer1(~(t_basico/2));
      tmr1_if=0;
      while(!tmr1_if);
      //rfin tem que ser '0'
      if(rfin) return;//erro
      set_timer1(~(t_basico));
      tmr1_if=0;
      while(!tmr1_if);
      //rfin poder ser '0' ou '1'
      buffer>>=1;
      if(buffer1&1)buffer|=0x80000000;
      buffer1>>=1;
      if(!rfin)buffer1|=0x80000000;


      set_timer1(~(t_basico));
      tmr1_if=0;
      while(!tmr1_if);
      //rfin tem que ser '1'
      if(!rfin){return;}//erro

      set_timer1(~(t_basico));
      tmr1_if=0;
      while(!tmr1_if&&rfin);//aguarda entrada rfin ficar '0'
      if(tmr1_if) return; //erro; excessivamente alto >65ms
      //rfin=0;
  }while(--qb);//recebe todos os 56 bits do tx


        flag_rok=1;//tem uma recepo no buffer aguardando ser tratada

}//VOID isr()



//==============================================================================
//   FIM   DA   ROTINA DE INTERRUPO DO TIMER 0
//==============================================================================

//==============================================================================
///////////////////////////////////////////////////////////////////////////////
//                    ROTINA PRINCIPAL
////////////////////////////////////////////////////////////////////////////////
//==============================================================================

//==============================================================================
//   INICIALIZAO DE PORTAS E REGISTRADORES
//==============================================================================

void main(){
   #ifdef  F675
   setup_adc_ports(no_analogs);  //somente pic12f675
   SETUP_ADC(ADC_OFF); //DESLIGA ADC
   #endif
   setup_comparator(nc_nc_nc_nc); // desliga comparadores
   setup_counters (RTCC_INTERNAL,RTCC_DIV_1);   //timer 0
   setup_timer_1 ( T1_INTERNAL | T1_DIV_BY_1 );
   ENABLE_INTERRUPTS(INT_RA3); // liga interrupo por mudana
   enable_interrupts(GLOBAL);   //  liga interrupo geral
   PORT_A_PULLUPS(TRUE);//pull up geral
   wpu=0xff;//liga pull up individual
   flag_rok=0;
   flag_gr=0;
   TRISIO= 0b011101;// ajusta trisio
   gpio=0;
   delay_ms(10);
   if(jump) s0=read_eeprom(127);//retorna situao da sada no reset
//==============================================================================
//   LOOP  PRINCIPAL DE REPETIO
//==============================================================================

   while (TRUE) {

    int8 a,b,c;


//==============================================================================
//   TESTA BOTO 'PROG' SE FOI APERTADO
//==============================================================================


       if(!bot){
            tm3=0;    //reseta contadores de apagamento do led 'prog' em
            tm2=0;    //modo de gravao de tx
            tm1=0;
            flag_gr=1; //aciona flag p/ gravao (ir ligar led 'prog')
            for (a=0;!bot;a++) {
               delay_ms(50);
               if(a==100){
                  apagar_eeprom();
                  flag_gr=0;
                  for(a=0;a<5;a++){
                     LED=1;
                     delay_ms(250);
                     LED=0;
                     delay_ms(250);
                     }
                   break;
                  }
            }
        }
 //=============================================================================
 // liga led 'learn' em modo de gravao de TX
 //=============================================================================

       if(flag_gr){ LED=1;}  else { LED=0; }//
//==============================================================================
//        TESTA CHECK SUN
//==============================================================================
       if(flag_rok){

       a=&buffer+1;//aponta inicio do dado
       b=0;//zera
       for(c=0;c<6;c++){//pega 6 bytes somando em b
         b+=*a;
         a++;
       }
       a=buffer1>>24;//pega check sun recebido no 7 byte

       //compara soma de bytes com check sun recebido
       if(a==b)  acionar();  //se iguais,  recepo correta, aciona a sada
       flag_rok=0;//autoriza  recepo

       }

//==============================================================================

//==============================================================================
//     TEMPO QUE FICA ACESO O LED 'PROG' AO APERTAR BOTO 'PROG'
//==============================================================================

   if(++tm1==0){ if(++tm2==0) { if(++tm3>=TMAX_LED) flag_gr=0; } }

//==============================================================================
//     TEMPO PARA DESLIGAMENTO DA SAIDA EM MODO PULSO
//==============================================================================

     if(++tl1==0){
       if(++tl2==TMAX_SAIDAS_LIGADAS) {
        tl2=0;
        tl1=0;
        if(jump){flag_s0=0; }//desliga flags0 em modo reteno
        else{ s0=0;} //desliga s0 em modo pulso
       }
      }

//==============================================================================
   }// while (TRUE)
}//void main()

//------------------FIM------------------------------------------------------
